#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>
#include <float.h>

static int convertflag = 1;

int hps_convert (int flag)
{
	convertflag = flag;

	return (HPS_OK);
}

int hps_get_convert ()
{
	return (convertflag);
}

LONGLONG BCD2val (unsigned addr, int size, int base)
{
	int ind;
	LONGLONG val = 0;

	val = 0;
	for (ind = size - 1; ind >= 0; ind--)
	{
		int digit;

		// Gets the digit
		digit = sat_peek (addr + ind, 1);
		// Adds it to the current value
		val = val*base + digit;
	}

	return (val);
}

void val2BCD (LONGLONG val, unsigned addr, int size, int base)
{
	int ind;

	for (ind = 0; ind < size; ind++)
	{
		sat_poke (addr + ind, val % base, 1);
		val /= base;
	}
}

double real2double (unsigned address)
{
	LONGLONG exp;
	LONGLONG mantissa;
	int sign;
	double val = 0.;

	// Gets the exponent
	exp = BCD2val (address, 3, 10);
	// Signs it
	if (exp > 499)
	{
		exp -= 1000;
	}

	// Gets the mantissa
	mantissa = BCD2val (address + 3, 12, 10);

	// Gets the sign
	sign = BCD2val (address + 15, 1, 10);

	// Computes the result
	val = (double) mantissa / 1E11 * pow (10, exp) * ((sign == 9) ? -1 : 1);

	return (val);
}

void double2real (double val, unsigned address)
{
	int sign;
	LONGLONG exp;
	LONGLONG mantissa;

	// Computes the sign
	sign = (val < 0) ? 9 : 0;

	// Keeps only absolute value for further computations
	val = fabs(val);
	
	// Computes the exponent
	exp = (int) floor (log10 (val));

	// Computes the mantissa
	mantissa = (LONGLONG) round ((val / pow (10, exp)) * 1E11);
	if (mantissa >= 1E12)
	{
		mantissa /= 10;
		exp++;
	}

	// Signs the exponent
	if (exp < 0)
	{
		exp += 1000;
	}

	// Fills the exponent
	val2BCD (exp, address, 3, 10);

	// Fills the mantissa
	val2BCD (mantissa, address + 3, 12, 10);

	// Fills the sign
	sat_poke (address + 15, sign, 1);
}

// Converts raw RPL type into target C type
int hps_convert_type (int type)
{
	int type_out;

	switch (type)
	{
		case xNEG :
		case xNEGNEG :
		case xABS :
		case xPLUS :
		case xMINUS :
		case xMULTIPLY :
		case xDIVIDE :
		case xPOW :
		case xROOT :
		case xSIGN :
		case xSQRT :
		case xSQ	:
		case xSIN :
		case xCOS :
		case xTAN :
		case xASIN :
		case xACOS :
		case xATAN :
		case xEXP :
		case xLN	:
		case xLOG :
		case xALOG :
		case xFACT :
		case xIP :
		case xFP :
		case xEQ :
		case xDIFF :
		case xLT :
		case xGT :
		case xLE :
		case xGE :
		case xSUM :
		{
			type_out = DOFUNC;
			break;
		}

		case ZINT_m9 :
		case ZINT_m8 :
		case ZINT_m7 :
		case ZINT_m6 :
		case ZINT_m5 :
		case ZINT_m4 :
		case ZINT_m3 :
		case ZINT_m2 :
		case ZINT_m1 :
		case ZINT_0 :
		case ZINT_1 :
		case ZINT_2 :
		case ZINT_3 :
		case ZINT_4 :
		case ZINT_5 :
		case ZINT_6 :
		case ZINT_7 :
		case ZINT_8 :
		case ZINT_9 :
		case ZINT_10 :
		{
			type_out = DOINT;
			break;
		}

		case REAL_m9 :
		case REAL_m8 :
		case REAL_m7 :
		case REAL_m6 :
		case REAL_m5 :
		case REAL_m4 :
		case REAL_m3 :
		case REAL_m2 :
		case REAL_m1 :
		case REAL_0 :
		case REAL_1 :
		case REAL_2 :
		case REAL_3 :
		case REAL_4 :
		case REAL_5 :
		case REAL_6 :
		case REAL_7 :
		case REAL_8 :
		case REAL_9 :
		case MAXREAL :
		case xPI :
		case xe :
		{
			type_out = DOREAL;
			break;
		}

		case DOARRY :
		{
			type_out = DOMATRIX;
			break;
		}
		
		default :
		{
			type_out = type;
			break;
		}
	}

	return (type_out);
}

int hps_get_type (int level, int *type)
{
	int Ret = HPS_OK;

	if (type)
	{
		int depth;

		// Gets the stack depth
		(void) hps_depth (&depth);

		// Checks that the asked level exists
		if ((level > 0) && (level <= depth))
		{
			// Gets the address of the object located at this level
			unsigned address = sat_peek (sat_peek (SAT_DSKTOP, 5) + (level-1) * 5, 5);
			int prolog = sat_peek (address, 5);

			*type = hps_convert_type (prolog);
		}
		else
		{
			Ret = HPS_ERROR_STACK;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}

	return (Ret);
}
