#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_decode_int (unsigned address, hpobj_t *obj, int *incr)
{
	int Ret = HPS_OK;
	unsigned prolog;

	if (obj && incr)
	{
		if (obj->type)
		{
			prolog = obj->type;							// There is no prologue (used by DOARRY)
			*incr = 0;
		}
		else
		{
			prolog = sat_peek (address, 5);
			address += 5;									// Skips the prologue
			*incr = 5;
		}

		switch (prolog)
		{
			case ZINT_m9 :
			{
				obj->type = DOINT;
				obj->data.doint = -9;
				break;
			}

			case ZINT_m8 :
			{
				obj->type = DOINT;
				obj->data.doint = -8;
				break;
			}

			case ZINT_m7 :
			{
				obj->type = DOINT;
				obj->data.doint = -7;
				break;
			}

			case ZINT_m6 :
			{
				obj->type = DOINT;
				obj->data.doint = -6;
				break;
			}

			case ZINT_m5 :
			{
				obj->type = DOINT;
				obj->data.doint = -5;
				break;
			}

			case ZINT_m4 :
			{
				obj->type = DOINT;
				obj->data.doint = -4;
				break;
			}

			case ZINT_m3 :
			{
				obj->type = DOINT;
				obj->data.doint = -3;
				break;
			}

			case ZINT_m2 :
			{
				obj->type = DOINT;
				obj->data.doint = -2;
				break;
			}

			case ZINT_m1 :
			{
				obj->type = DOINT;
				obj->data.doint = -1;
				break;
			}

			case ZINT_0 :
			{
				obj->type = DOINT;
				obj->data.doint = 0;
				break;
			}

			case ZINT_1 :
			{
				obj->type = DOINT;
				obj->data.doint = 1;
				break;
			}

			case ZINT_2 :
			{
				obj->type = DOINT;
				obj->data.doint = 2;
				break;
			}

			case ZINT_3 :
			{
				obj->type = DOINT;
				obj->data.doint = 3;
				break;
			}

			case ZINT_4 :
			{
				obj->type = DOINT;
				obj->data.doint = 4;
				break;
			}

			case ZINT_5 :
			{
				obj->type = DOINT;
				obj->data.doint = 5;
				break;
			}

			case ZINT_6 :
			{
				obj->type = DOINT;
				obj->data.doint = 6;
				break;
			}

			case ZINT_7 :
			{
				obj->type = DOINT;
				obj->data.doint = 7;
				break;
			}

			case ZINT_8 :
			{
				obj->type = DOINT;
				obj->data.doint = 8;
				break;
			}

			case ZINT_9 :
			{
				obj->type = DOINT;
				obj->data.doint = 9;
				break;
			}

			case ZINT_10 :
			{
				obj->type = DOINT;
				obj->data.doint = 10;
				break;
			}

			case DOINT :		// ZINT -> long long
			{
				// Gets size of int
				unsigned int nb = sat_peek (address, 5) - 5;

				// Checks for overflow
				if (nb < 19)				// long long max = 1.844 e19
				{
					int sign;
					LONGLONG val = 0;

					// Gets the value
					val = BCD2val (address + 5, nb - 1, 10);

					// Gets the sign
					sign = sat_peek (address + 5 + nb - 1, 1);
					if (sign == 9)
					{
						val = -val;
					}

					// Sets the output parameters
					obj->type = DOINT;
					obj->data.doint = val;
				}
				else
				{
					Ret = HPS_ERROR_OVERFLOW;
				}

				*incr += 5 + nb;		// size + nb digits = sign
				break;
			}

			default :
			{
				Ret = HPS_ERROR_TYPE;
				break;
			}
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}

	return (Ret);
}
