#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_obj_create (hpobj_t **obj)
{	
	int Ret = HPS_OK;
	
	if ((*obj = (hpobj_t *) malloc (sizeof (hpobj_t))) == NULL)
	{
		Ret = HPS_ERROR_ARM_MEMORY;
	}
	
	return (Ret);
}

int hps_obj_destroy (hpobj_t **obj)
{
	int Ret = HPS_OK;
	
	if (obj && *obj)
	{
		free (*obj);
		obj = NULL;
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_obj_destroy_all (hpobj_t **obj)
{
	int Ret = HPS_OK;

	if (obj && *obj)
	{
		switch ((*obj)->type)
		{
			case DOCSTR :
			{
				char *str = (*obj)->data.docstr;
				
				if (str)
				{
					free (str);
				}
				
				break;
			}
				
			case DOLIST :
			{
				list_t *list = (*obj)->data.dolist;
				
				if (list)
				{
					(void) hps_list_destroy (&list);
				}
				
				break;
			}
			
			case DOMATRIX :
			{
				array_t *array = (*obj)->data.domatrix;
				
				if (array)
				{
					(void) hps_array_destroy (&array);
				}
					break;
			}
		}
		
		Ret = hps_obj_destroy (obj);
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_obj_get (hpobj_t *obj, int *type, void **data)
{
	int Ret = HPS_OK;
	
	if (obj && type && data)
	{
		*type = obj->type;
		*data = (void *) &(obj->data);
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_obj_create_type (int type, void *data, hpobj_t **obj)
{
	int Ret = HPS_OK;
	
	if (data && obj)
	{
		if ((Ret = hps_obj_create (obj)) == HPS_OK)
		{
			(*obj)->type = type;
			(void) memcpy ((void *) &((*obj)->data), data, sizeof ((*obj)->data));
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_obj_create_int (LONGLONG val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOINT, (void *) &val, obj);
	
	return (Ret);
}

int hps_obj_create_uint (ULONGLONG val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOHXS, (void *) &val, obj);
	
	return (Ret);
}

int hps_obj_create_real (double val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOREAL, (void *) &val, obj);
	
	return (Ret);
}

int hps_obj_create_complex (complex_t *val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOCMP, (void *) val, obj);
	
	return (Ret);
}

int hps_obj_create_str (char *val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOCSTR, (void *) &val, obj);
	
	return (Ret);
}

int hps_obj_create_list (list_t *val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOLIST, (void *) &val, obj);
	
	return (Ret);
}

int hps_obj_create_array (array_t *val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOMATRIX, (void *) &val, obj);
	
	return (Ret);
}

int hps_obj_create_ident (char *val, hpobj_t **obj)
{
	int Ret;
	
	Ret = hps_obj_create_type (DOIDNT, (void *) &val, obj);
	
	return (Ret);
}
